pro readtables, filename, directory, ObsList, FRMList, FDBList, CRSList, error, copyxml, destination

  ; procedure to read the xml-files from Ankurs planning tool
  ; INPUT:
  ;     filename: the name of the Observing List Table
  ;             the other filenames of FRM, FDB and CRS will be taken from there, from the IDs
  ;     directory: the directory, in which all xml-files are in
  ;     destination: the folder in which all results are saved, this procedure will copy the xml-files there
  ;     copyxml: 1 if xml-files should be copied to destination, 0 if not
  ;
  ; OUTPUT:
  ;     ObsList: a structure, derived from iris_structure.pro containing
  ;           all variables of the Observing list table
  ;     FRMList: an array of structures, containing the FRMs
  ;     FDBList: an array of structures, containing the FDBs
  ;     CRSList: an array of structures, containing the CRSs
  ;     error: is 1, if an error occurred
  ;
  ; MODIFICATION HISTORY:
  ; written by Martin Wiesmann (ITA, UIO), 5.6.2012
  ; modified it, so that xml-files will be copied to a destination-folder, 30.8.2012
  

  defaultlocation='XMLTest/'
  defaultfilename='OBS-T-00001.xml'
  
  error=0
  
  if N_PARAMS() eq 0 then filename=defaultfilename
  if N_PARAMS() le 1 then directory=defaultlocation
  sfile = size(filename)
  if sfile[N_ELEMENTS(sfile)-2] ne 7 then begin
    print, 'first parameter (filename) must be a string'
    error=1
    return
  endif
  if filename eq '' then filename=defaultfilename
  file=directory+filename
  if ~FILE_TEST(file) then begin
    print, 'invalid filename, could not load OBS file: ' + file
    error=1
    return
  endif
  
  
  ;copy OBS-table
  if copyxml then file_copy,file,destination
  
  ;read OBS-table
  listObject = OBJ_NEW('pt_xml2struct') ;create a pt_xml2struct object, which inherits IDLffXMLSAX
  listObject->parseFile, file       ;read and parse file
  ObsList = listObject->getArray()      ;get structure with observation list table
  OBJ_DESTROY, listObject               ;destroy object
  
  print, 'Observing list table loaded, ID: ' + string(ObsList.ID) + '; ' + string(ObsList.NumEntries) + ' entries'
  
  ;get FRM-IDs
  if ObsList.NumEntries gt 0 then begin
    for i=0,ObsList.NumEntries-1 do begin
      if i eq 0 then FRMids=(*(ObsList).FRM_ID)[i] else begin
        index = WHERE(FRMids eq (*(ObsList).FRM_ID)[i],count)
        if count eq 0 then FRMids = [FRMids, (*(ObsList).FRM_ID)[i]]
      endelse
    endfor
    print, string(N_ELEMENTS(FRMids)) + ' FRM-IDs found'
    print, 'IDs: ', FRMids
    
    
    
    ;read FRM-tables
    FDBidstemp=0
    for nfrm=0,N_ELEMENTS(FRMids)-1 do begin
      filename='FRM-'+FRMids[nfrm]+'.xml'
      file=directory+filename
      if ~FILE_TEST(file) then begin
        print, 'invalid filename, could not load FRM file: ' + file
        error=1
        return
      end
      
      ;copy FRM-table
      if copyxml then file_copy,file,destination
      
      listObject = OBJ_NEW('pt_xml2struct')
      listObject->parseFile, file
      FRMListtemp = listObject->getArray()
      OBJ_DESTROY, listObject
      
      print, 'Frame List Table loaded, ID: ' + string(FRMListtemp.ID) + '; ' + string(FRMListtemp.NumEntries) + ' entries'
      
      ;create an array of FRM-tables
      if nfrm eq 0 then begin
        FRMList = MAKE_ARRAY(N_ELEMENTS(FRMids), Value=FRMListtemp)
      endif else begin
        FRMList[nfrm] = FRMListtemp
      endelse
      
      ; temporary variable, max number of FDB IDs
      FDBidstemp = FDBidstemp + 3 * FRMListtemp.NumEntries
    endfor ;read FRM-tables
    
    ;Get number of FDB IDs and their IDs
    FDBids = MAKE_ARRAY(FDBidstemp, /String)
    k=0
    for i=0,N_ELEMENTS(FRMids)-1 do begin
      for j=0,FRMList[i].NumEntries-1 do begin
        FDBids[k]=(*(FRMList[i]).SJI_FDB_ID)[j]
        k=k+1
        FDBids[k]=(*(FRMList[i]).NUV_SG_FDB_ID)[j]
        k=k+1
        FDBids[k]=(*(FRMList[i]).FUV_SG_FDB_ID)[j]
        k=k+1
      endfor
    endfor
    index = WHERE(FDBids ne '0', count)
    if count eq 0 then begin
      print, 'No FDBs defined'
      return
    endif
    FDBids = FDBids[index]
    
    if N_ELEMENTS(FDBids) gt 1 then begin
      temp = FDBids
      for i=0,N_ELEMENTS(temp)-1 do begin
        if i eq 0 then FDBids = temp[i] else begin
          index = WHERE(temp[i] eq FDBids,count)
          if count eq 0 then FDBids = [FDBids, temp[i]]
        endelse
      endfor
    endif
    print, string(N_ELEMENTS(FDBids)) + ' FDB-IDs found'
    print, 'IDs: ', FDBids
    
    
    if N_ELEMENTS(FDBids) gt 0 then begin
    
    
    
      ;read FDB-table
      for nfdb=0,N_ELEMENTS(FDBids)-1 do begin
        filename='FDB-'+FDBids[nfdb]+'.xml'
        file=directory+filename
        if ~FILE_TEST(file) then begin
          print, 'invalid filename, could not load FDB file: ' + file
          error=1
          return
        end
        
        ;copy FDB-table
        if copyxml then file_copy,file,destination
        
        listObject = OBJ_NEW('pt_xml2struct')
        listObject->parseFile, file
        FDBListtemp = listObject->getArray()    ;get structure with observation list table
        OBJ_DESTROY, listObject
        
        print, 'Frame Definition Block Table loaded, ID: ' + string(FDBListtemp.ID)
        
        ;create an array of FDB-tables
        if nfdb eq 0 then begin
          FDBList = MAKE_ARRAY(N_ELEMENTS(FDBids), Value=FDBListtemp)
        endif else begin
          FDBList[nfdb] = FDBListtemp
        endelse
      endfor ;read FDB-table
      
      ;get CRS-IDs
      for i=0,N_ELEMENTS(FDBids)-1 do begin
        if i eq 0 then begin
          CRSids = FDBlist[i].CRSID
          type = FDBlist[i].type
        endif else begin
          index = WHERE(FDBlist[i].CRSID eq CRSids,count)
          if count eq 0 then begin
            CRSids = [CRSids, FDBlist[i].CRSID]
            type = [type, FDBlist[i].type]
          endif
        endelse
      endfor
      ;endif
      print, string(N_ELEMENTS(CRSids)) + ' CRS-IDs found'
      print, 'IDs: ', CRSids
      print, 'Types: ', type
      
      
      
      if N_ELEMENTS(CRSids) gt 0 then begin
      
      
      
        ;read CRS-table
        for ncrs=0,N_ELEMENTS(CRSids)-1 do begin
          filename='CRS-'+CRSids[ncrs]+'-'+type[ncrs]+'.xml'
          file=directory+filename
          if ~FILE_TEST(file) then begin
            print, 'invalid filename, could not load CRS file: ' + file
            error=1
            return
          end
          
          ;copy CRS-table
          if copyxml then file_copy,file,destination
          
          listObject = OBJ_NEW('pt_xml2struct')
          listObject->parseFile, file
          CRSListtemp = listObject->getArray()    ;get structure with observation list table
          OBJ_DESTROY, listObject
          
          print, 'CCD Readout Spec Table loaded, ID: ' + string(CRSListtemp.ID) + '; ' + string(CRSListtemp.SubRegions) + ' subregions'
          
          ;help,CRSListtemp
          
          ;create an array of FDB-tables
          if ncrs eq 0 then begin
            CRSList = MAKE_ARRAY(N_ELEMENTS(CRSids), Value=CRSListtemp)
          endif else begin
            CRSList[ncrs] = CRSListtemp
          endelse
        endfor ;read CRS-table
        
      endif else error=1 ;N_ELEMENTS(CRSids) gt 0
    endif else error=1 ;N_ELEMENTS(FDBids) gt 0
  endif else error=1 ;ObsList.NumEntries gt 0
  
end